@@ -22,7 +22,7 @@ use Carp;
 use Excel::Writer::XLSX::Chart;
 
 our @ISA     = qw(Excel::Writer::XLSX::Chart);
-our $VERSION = '0.79';
+our $VERSION = '0.80';
 
 
 ###############################################################################
@@ -46,6 +46,10 @@ sub new {
 
     $self->set_y_axis();
 
+    # Sset the available data label positions for this chart type.
+    $self->{_label_position_default} = 'center';
+    $self->{_label_positions} = { center => 'ctr' };
+
     bless $self, $class;
     return $self;
 }
