@@ -22,7 +22,7 @@ use Carp;
 use Excel::Writer::XLSX::Chart;
 
 our @ISA     = qw(Excel::Writer::XLSX::Chart);
-our $VERSION = '0.78';
+our $VERSION = '0.80';
 
 
 ###############################################################################
@@ -48,6 +48,10 @@ sub new {
     # Hardcode major_tick_mark for now until there is an accessor.
     $self->{_y_axis}->{_major_tick_mark} = 'cross';
 
+    # Set the available data label positions for this chart type.
+    $self->{_label_position_default} = 'center';
+    $self->{_label_positions} = { center => 'ctr' };
+
     bless $self, $class;
 
     return $self;
