@@ -33,7 +33,7 @@ use Excel::Writer::XLSX::Package::XMLwriter;
 use Excel::Writer::XLSX::Utility qw(xl_cell_to_rowcol xl_rowcol_to_cell);
 
 our @ISA     = qw(Excel::Writer::XLSX::Package::XMLwriter);
-our $VERSION = '0.78';
+our $VERSION = '0.80';
 
 
 ###############################################################################
@@ -54,8 +54,8 @@ sub new {
     my $class = shift;
     my $self  = Excel::Writer::XLSX::Package::XMLwriter->new();
 
-    $self->{_filename}           = $_[0] || '';
-    my $options                  = $_[1] || {};
+    $self->{_filename} = $_[0] || '';
+    my $options = $_[1] || {};
 
     $self->{_tempdir}            = undef;
     $self->{_date_1904}          = 0;
@@ -523,7 +523,7 @@ sub add_format {
     }
 
     # Add the default format properties.
-    push @init_data, %{$self->{_default_format_properties}};
+    push @init_data, %{ $self->{_default_format_properties} };
 
     # Add the user defined properties.
     push @init_data, @_;
@@ -845,7 +845,7 @@ sub set_properties {
 #
 sub add_vba_project {
 
-    my $self = shift;
+    my $self        = shift;
     my $vba_project = shift;
 
     croak "No vbaProject.bin specified in add_vba_project()"
@@ -1516,34 +1516,91 @@ sub _prepare_drawings {
         my $chart_count = scalar @{ $sheet->{_charts} };
         my $image_count = scalar @{ $sheet->{_images} };
         my $shape_count = scalar @{ $sheet->{_shapes} };
-        next unless ( $chart_count + $image_count + $shape_count );
 
-        $drawing_id++;
+        my $header_image_count = scalar @{ $sheet->{_header_images} };
+        my $footer_image_count = scalar @{ $sheet->{_footer_images} };
+        my $has_drawing        = 0;
 
+
+        # Check that some image or drawing needs to be processed.
+        if (   !$chart_count
+            && !$image_count
+            && !$shape_count
+            && !$header_image_count
+            && !$footer_image_count )
+        {
+            next;
+        }
+
+        # Don't increase the drawing_id header/footer images.
+        if ( $chart_count || $image_count || $shape_count ) {
+            $drawing_id++;
+            $has_drawing = 1;
+        }
+
+        # Prepare the worksheet charts.
         for my $index ( 0 .. $chart_count - 1 ) {
             $chart_ref_id++;
             $sheet->_prepare_chart( $index, $chart_ref_id, $drawing_id );
         }
 
+        # Prepare the worksheet images.
         for my $index ( 0 .. $image_count - 1 ) {
 
             my $filename = $sheet->{_images}->[$index]->[2];
 
-            my ( $image_id, $type, $width, $height, $name ) =
+            my ( $type, $width, $height, $name, $x_dpi, $y_dpi ) =
               $self->_get_image_properties( $filename );
 
             $image_ref_id++;
 
-            $sheet->_prepare_image( $index, $image_ref_id, $drawing_id, $width,
-                $height, $name, $type );
+            $sheet->_prepare_image(
+                $index, $image_ref_id, $drawing_id,
+                $width, $height,       $name,
+                $type,  $x_dpi,        $y_dpi
+            );
         }
 
+        # Prepare the worksheet shapes.
         for my $index ( 0 .. $shape_count - 1 ) {
             $sheet->_prepare_shape( $index, $drawing_id );
         }
 
-        my $drawing = $sheet->{_drawing};
-        push @{ $self->{_drawings} }, $drawing;
+        # Prepare the header images.
+        for my $index ( 0 .. $header_image_count - 1 ) {
+
+            my $filename = $sheet->{_header_images}->[$index]->[0];
+            my $position = $sheet->{_header_images}->[$index]->[1];
+
+            my ( $type, $width, $height, $name, $x_dpi, $y_dpi ) =
+              $self->_get_image_properties( $filename );
+
+            $image_ref_id++;
+
+            $sheet->_prepare_header_image( $image_ref_id, $width, $height,
+                $name, $type, $position, $x_dpi, $y_dpi );
+        }
+
+        # Prepare the footer images.
+        for my $index ( 0 .. $footer_image_count - 1 ) {
+
+            my $filename = $sheet->{_footer_images}->[$index]->[0];
+            my $position = $sheet->{_footer_images}->[$index]->[1];
+
+            my ( $type, $width, $height, $name, $x_dpi, $y_dpi ) =
+              $self->_get_image_properties( $filename );
+
+            $image_ref_id++;
+
+            $sheet->_prepare_header_image( $image_ref_id, $width, $height,
+                $name, $type, $position, $x_dpi, $y_dpi );
+        }
+
+
+        if ( $has_drawing ) {
+            my $drawing = $sheet->{_drawing};
+            push @{ $self->{_drawings} }, $drawing;
+        }
     }
 
     # Sort the workbook charts references into the order that the were
@@ -1571,25 +1628,39 @@ sub _prepare_vml_objects {
     my $comment_id     = 0;
     my $vml_drawing_id = 0;
     my $vml_data_id    = 1;
+    my $vml_header_id  = 0;
     my $vml_shape_id   = 1024;
     my $vml_files      = 0;
     my $comment_files  = 0;
 
     for my $sheet ( @{ $self->{_worksheets} } ) {
 
-        next unless $sheet->{_has_vml};
-        $vml_files++;
-        $comment_files++ if $sheet->{_has_comments};
-        $comment_id++    if $sheet->{_has_comments};
-        $vml_drawing_id++;
+        next if !$sheet->{_has_vml} and !$sheet->{_has_header_vml};
+        $vml_files = 1;
 
-        my $count =
-          $sheet->_prepare_vml_objects( $vml_data_id, $vml_shape_id,
-            $vml_drawing_id, $comment_id );
 
-        # Each VML file should start with a shape id incremented by 1024.
-        $vml_data_id  += 1 * int(    ( 1024 + $count ) / 1024 );
-        $vml_shape_id += 1024 * int( ( 1024 + $count ) / 1024 );
+        if ( $sheet->{_has_vml} ) {
+
+            $comment_files++ if $sheet->{_has_comments};
+            $comment_id++    if $sheet->{_has_comments};
+            $vml_drawing_id++;
+
+            my $count =
+              $sheet->_prepare_vml_objects( $vml_data_id, $vml_shape_id,
+                $vml_drawing_id, $comment_id );
+
+            # Each VML file should start with a shape id incremented by 1024.
+            $vml_data_id  += 1 * int(    ( 1024 + $count ) / 1024 );
+            $vml_shape_id += 1024 * int( ( 1024 + $count ) / 1024 );
+
+        }
+
+        if ( $sheet->{_has_header_vml} ) {
+            $vml_header_id++;
+            $vml_drawing_id++;
+            $sheet->_prepare_header_vml_objects( $vml_header_id,
+                $vml_drawing_id );
+        }
     }
 
     $self->{_num_vml_files}     = $vml_files;
@@ -1840,80 +1911,59 @@ sub _get_image_properties {
     my $self     = shift;
     my $filename = shift;
 
-    my %images_seen;
-    my @image_data;
-    my @previous_images;
-    my $image_id = 1;
     my $type;
     my $width;
     my $height;
+    my $x_dpi = 96;
+    my $y_dpi = 96;
     my $image_name;
 
-    if ( not exists $images_seen{$filename} ) {
 
-        ( $image_name ) = fileparse( $filename );
+    ( $image_name ) = fileparse( $filename );
 
-        # TODO should also match seen images based on checksum.
+    # Open the image file and import the data.
+    my $fh = FileHandle->new( $filename );
+    croak "Couldn't import $filename: $!" unless defined $fh;
+    binmode $fh;
 
-        # Open the image file and import the data.
-        my $fh = FileHandle->new( $filename );
-        croak "Couldn't import $filename: $!" unless defined $fh;
-        binmode $fh;
+    # Slurp the file into a string and do some size calcs.
+    my $data = do { local $/; <$fh> };
+    my $size = length $data;
 
-        # Slurp the file into a string and do some size calcs.
-        my $data = do { local $/; <$fh> };
-        my $size = length $data;
 
-        #my $checksum   = TODO.
+    if ( unpack( 'x A3', $data ) eq 'PNG' ) {
 
-        if ( unpack( 'x A3', $data ) eq 'PNG' ) {
+        # Test for PNGs.
+        ( $type, $width, $height, $x_dpi, $y_dpi ) =
+          $self->_process_png( $data, $filename );
 
-            # Test for PNGs.
-            ( $type, $width, $height ) = $self->_process_png( $data );
-            $self->{_image_types}->{png} = 1;
-        }
-        elsif ( unpack( 'n', $data ) == 0xFFD8 ) {
+        $self->{_image_types}->{png} = 1;
+    }
+    elsif ( unpack( 'n', $data ) == 0xFFD8 ) {
 
-            # Test for JPEG files.
-            ( $type, $width, $height ) =
-              $self->_process_jpg( $data, $filename );
+        # Test for JPEG files.
+        ( $type, $width, $height, $x_dpi, $y_dpi ) =
+          $self->_process_jpg( $data, $filename );
 
-            $self->{_image_types}->{jpeg} = 1;
+        $self->{_image_types}->{jpeg} = 1;
+    }
+    elsif ( unpack( 'A2', $data ) eq 'BM' ) {
 
-        }
-        elsif ( unpack( 'A2', $data ) eq 'BM' ) {
+        # Test for BMPs.
+        ( $type, $width, $height ) = $self->_process_bmp( $data, $filename );
 
-            # Test for BMPs.
-            ( $type, $width, $height ) =
-              $self->_process_bmp( $data, $filename );
-
-            $self->{_image_types}->{bmp} = 1;
-        }
-        else {
-
-            # TODO. Add Image::Size to support other types.
-            croak "Unsupported image format for file: $filename\n";
-        }
-
-        push @{ $self->{_images} }, [ $filename, $type ];
-
-        # Also store new data for use in duplicate images.
-        push @previous_images, [ $image_id, $type, $width, $height ];
-        $images_seen{$filename} = $image_id++;
-
-        $fh->close;
+        $self->{_image_types}->{bmp} = 1;
     }
     else {
-
-        # We've processed this file already.
-        my $index = $images_seen{$filename} - 1;
-
-        # Increase image reference count.
-        $image_data[$index]->[0]++;
-
+        croak "Unsupported image format for file: $filename\n";
     }
 
-    return ( $image_id, $type, $width, $height, $image_name );
+    push @{ $self->{_images} }, [ $filename, $type ];
+
+
+    $fh->close;
+
+    return ( $type, $width, $height, $image_name, $x_dpi, $y_dpi );
 }
 
 
@@ -1925,13 +1975,52 @@ sub _get_image_properties {
 #
 sub _process_png {
 
-    my $self = shift;
+    my $self     = shift;
+    my $data     = $_[0];
+    my $filename = $_[1];
 
     my $type   = 'png';
-    my $width  = unpack "N", substr $_[0], 16, 4;
-    my $height = unpack "N", substr $_[0], 20, 4;
+    my $width  = 0;
+    my $height = 0;
+    my $x_dpi  = 96;
+    my $y_dpi  = 96;
 
-    return ( $type, $width, $height );
+    my $offset      = 8;
+    my $data_length = length $data;
+
+    # Search through the image data to read the height and width in the
+    # IHDR element. Also read the DPI in the pHYs element.
+    while ( $offset < $data_length ) {
+
+        my $length = unpack "N",  substr $data, $offset + 0, 4;
+        my $type   = unpack "A4", substr $data, $offset + 4, 4;
+
+        if ( $type eq "IHDR" ) {
+            $width  = unpack "N", substr $data, $offset + 8,  4;
+            $height = unpack "N", substr $data, $offset + 12, 4;
+        }
+
+        if ( $type eq "pHYs" ) {
+            my $x_ppu = unpack "N", substr $data, $offset + 8,  4;
+            my $y_ppu = unpack "N", substr $data, $offset + 12, 4;
+            my $units = unpack "C", substr $data, $offset + 16, 1;
+
+            if ( $units == 1 ) {
+                $x_dpi = $x_ppu * 0.0254;
+                $y_dpi = $y_ppu * 0.0254;
+            }
+        }
+
+        $offset = $offset + $length + 12;
+
+        last if $type eq "IEND";
+    }
+
+    if ( not defined $height ) {
+        croak "$filename: no size data found in png image.\n";
+    }
+
+    return ( $type, $width, $height, $x_dpi, $y_dpi );
 }
 
 
@@ -2003,23 +2092,40 @@ sub _process_jpg {
     my $data     = $_[0];
     my $filename = $_[1];
     my $type     = 'jpeg';
+    my $x_dpi    = 96;
+    my $y_dpi    = 96;
     my $width;
     my $height;
 
     my $offset      = 2;
     my $data_length = length $data;
 
-    # Search through the image data to find the 0xFFC0 marker. The height and
-    # width are contained in the data for that sub element.
+    # Search through the image data to read the height and width in the
+    # 0xFFC0/C2 element. Also read the DPI in the 0xFFE0 element.
     while ( $offset < $data_length ) {
 
-        my $marker = unpack "n", substr $data, $offset, 2;
+        my $marker = unpack "n", substr $data, $offset + 0, 2;
         my $length = unpack "n", substr $data, $offset + 2, 2;
 
         if ( $marker == 0xFFC0 || $marker == 0xFFC2 ) {
             $height = unpack "n", substr $data, $offset + 5, 2;
             $width  = unpack "n", substr $data, $offset + 7, 2;
-            last;
+        }
+
+        if ( $marker == 0xFFE0 ) {
+            my $units     = unpack "C", substr $data, $offset + 11, 1;
+            my $x_density = unpack "n", substr $data, $offset + 12, 2;
+            my $y_density = unpack "n", substr $data, $offset + 14, 2;
+
+            if ( $units == 1 ) {
+                $x_dpi = $x_density;
+                $y_dpi = $y_density;
+            }
+
+            if ( $units == 2 ) {
+                $x_dpi = $x_density * 2.54;
+                $y_dpi = $y_density * 2.54;
+            }
         }
 
         $offset = $offset + $length + 2;
@@ -2030,7 +2136,7 @@ sub _process_jpg {
         croak "$filename: no size data found in jpeg image.\n";
     }
 
-    return ( $type, $width, $height );
+    return ( $type, $width, $height, $x_dpi, $y_dpi );
 }
 
 
@@ -2214,7 +2320,7 @@ sub _write_workbook_view {
     push @attributes, ( tabRatio => $tab_ratio ) if $tab_ratio != 500;
 
     # Store the firstSheet attribute when it isn't the default.
-    push @attributes, ( firstSheet => $first_sheet +1 ) if $first_sheet > 0;
+    push @attributes, ( firstSheet => $first_sheet + 1 ) if $first_sheet > 0;
 
     # Store the activeTab attribute when it isn't the first sheet.
     push @attributes, ( activeTab => $active_tab ) if $active_tab > 0;
